% inverted_pendulum.m

%% system parameters
M = .5;
m = 0.2;
b = 0.1;
I = 0.006;
g = 9.8;
l = 0.3;

%% Matrices for state equations
% x_t+1 = A*x_t + B*u
% y_t = C*x_t + D*u

p = I*(M+m)+M*m*l^2; %denominator for the A and B matrices

A = [0      1              0           0;
     0 -(I+m*l^2)*b/p  (m^2*g*l^2)/p   0;
     0      0              0           1;
     0 -(m*l*b)/p       m*g*l*(M+m)/p  0];
B = [     0;
     (I+m*l^2)/p;
          0;
        m*l/p];
C = [1 0 0 0;
     0 0 1 0];
D = [0;
     0];
 
%% simulate system
dT = 0.001;
x(:,1)=[0 0 0 0];
for i = 1:2000, 
    x(:,i+1)=(eye(4)+dT*A)*x(:,i)+0.000001*randn(4,1);
end
figure; plot(x')
